package ch.zhaw.ads;

import java.awt.*;
import java.util.Arrays;
import java.util.Vector;

public class LabyrinthServer implements CommandExecutor {
    ServerGraphics g = new ServerGraphics();

    public Graph<DijkstraNode, Edge> createGraph(String s) {
        Graph<DijkstraNode, Edge> graph = new AdjListGraph<>(DijkstraNode.class, Edge.class);
        Arrays.asList(s.split("\n")).forEach(s1 -> {
            try {
                String[] nodes = s1.split(" ");
                int x1 = Integer.parseInt(nodes[0].split("-")[0]);
                int x2 = Integer.parseInt(nodes[1].split("-")[0]);
                int y1 = Integer.parseInt(nodes[0].split("-")[1]);
                int y2 = Integer.parseInt(nodes[1].split("-")[1]);
                double distance = Math.abs(x1-x2) + Math.abs(y1-y2);
                graph.addEdge(nodes[0], nodes[1], distance);
                graph.addEdge(nodes[1], nodes[0], distance);
            } catch (Throwable e) {
                e.printStackTrace();
            }
        });
        return graph;
    }

    public void drawLabyrinth(Graph<DijkstraNode, Edge> graph) {
        // TODO implement 8.3
        g.setColor(Color.gray);
        g.fillRect(0, 0, 9, 9);
        g.setColor(Color.WHITE);
        graph.getNodes().forEach(node -> node.getEdges().forEach(edge -> g.drawPath(node.getName(), edge.dest.getName(), false)));
    }

    private boolean search(DijkstraNode current, DijkstraNode ziel) {
        current.setMark(true);
        for(Edge edge : current.getEdges()){
            DijkstraNode node = (DijkstraNode) edge.getDest();
            if(node == ziel || (!node.getMark() && search(node, ziel))){
                node.setPrev(current);
                return true;
            }
        }
        current.setMark(false);
        return false;
    }

    // search and draw result
    public void drawRoute(Graph<DijkstraNode, Edge> graph, String startNode, String zielNode) {
        search(graph.findNode(startNode), graph.findNode(zielNode));
        DijkstraNode currentNode = graph.findNode(zielNode);
        g.setColor(Color.red);
        while(currentNode.getPrev() != null){
            g.drawPath(currentNode.getName(), currentNode.getPrev().getName(), true);
            currentNode = currentNode.getPrev();
        }
    }

    public String execute(String s) {
        Graph<DijkstraNode, Edge> graph;
        graph = createGraph(s);
        drawLabyrinth(graph);
        drawRoute(graph, "0-6", "3-0");
        return g.getTrace();
    }
}
